;------------------------------
;; dark current subtraction
;------------------------------
;
function eis_cal::dc, data, iwin, dcfile=dcfile,verbose=verbose         ,init=init
  if n_elements(init) eq 1 then return,-1
; dark current subraction of window iwin from object data
  if n_elements(dcfile) ne 0 then self.dc_file=dcfile
  if n_elements(verbose) eq 0 then verbose=0
  if self.dc_file eq '' then begin  ; set to dc file name dc_defaultxxx.idlsave
    dirsep = get_delim()
    sdir = file_search(self.caldir,/mark_directory) ; append dirsep to end if not there already
    sdir = sdir+'dc'+dirsep+'dc_default*.idlsave'
    dclist = file_search(sdir,count = nfiles)
    if nfiles gt 0 then begin 
      self.dc_file = dclist[nfiles-1] 
    endif else begin
      message,'No dark current file found, using Q&D default routine',/info
      dc_data={version:'quickndirty'}
    endelse
  endif
  if iwin eq 0 then begin
    message,'using dark current file '+self.dc_file,/info
  endif
;
  data->getwin,iwin,wd,pos
  err=data->geterr(iwin)     ; PRY, 28-Aug-2019
  quad=self->quadrant(pos)
  aux_data=data->getaux_data()
;  exp_dur=((*aux_data.ti_2)-(*aux_data.ti_1))/512.
  exp_dur=data->getexp()
  restore,self.dc_file
  if datatype(dc_data) eq 'UND' then begin
    dc_data={version:'full_ccd'}
    if iwin eq 0 then begin
      message,'CAREFUL, full ccd dark current subtraction has no variation with',/info
      message,'         exposure time nor ccd temperature',/info
    endif
  endif

  missing=-100.
  
  case dc_data.version of
  
    'quickndirty': begin   

      slit_id = strtrim(data->getslit_id(),2)

      if slit_id eq '1"' or slit_id eq '2"' then begin
        sz = size(wd)
        if sz[1] eq 1024 then begin
          ;; --- full ccd, use low sensitivity areas. this is generally very accurate
          p1 = ([ 39, 944, 39, 926])[iwin] 
          p2 = ([ 84, 989, 84, 971])[iwin] 
          if iwin eq 0 or verbose then $
          message,"SLIT - FULL CCD, USING CONTINUUM REGION TO ESTIMATE THE BACKGROUND",/info
          for i=0,data->getnexp()-1 do begin
            wd_i = reform(wd[p1:p2,*,i])
            err_i= reform(err[p1:p2,*,i])   ; PRY, 28-Aug-2019
            k=where(err_i NE missing,nk)
            dc=0.
           ;
           ; PRY: it's possible that all the pixels in the
           ; sub-window are missing (nk=0) and so then the DC is
           ; computed using the 2% method.
           ;
            IF nk NE 0 THEN BEGIN
              dc   = median(wd_i[k])
            ENDIF ELSE BEGIN
              message,' -- CONTINUUM REGION MISSING, USING FULL WINDOW',/info
              wd_i=reform(wd[*,*,i])
              err_i= reform(err[*,*,i])
              k=where(err_i NE missing,nk)
              IF nk NE 0 THEN BEGIN 
                npx = max([0.02*nk,min([nk,100])])
                idx  = (sort(wd_i[k]))[0:npx-1]
                dc   = median(wd_i[k[idx]])
              ENDIF
            ENDELSE
            wd[*,*,i] = (wd[*,*,i] - dc); >0
            if verbose eq 1 then $
            message,"WINDOW = "+trim(iwin,'(i2.2)')+" NEXP = "+trim(i,'(i3.3)')+" DC = "+trim(dc,'(i3.3)'),/info
          endfor
        endif else begin
          ;; --- window, use the median of some low intensity pixels. this is generally an underestimate.
          if iwin eq 0 or verbose then $
             message,"SLIT - WINDOW, USING LOWEST 2% OF INTENSITIES TO ESTIMATE THE BACKGROUND",/info
         ;
         ; PRY, 17-Sep-2019: the following code using the pixels with
         ; the lowest 2% intensity to compute the DC. I've
         ; modified it so that it doesn't use missing pixels in
         ; the calculation. 
         ;
;          npx = 0.02*float(sz[1])*float(sz[2]) 
          for i=0,data->getnexp()-1 do BEGIN
            wd_i=reform(wd[*,*,i])
            err_i= reform(err[*,*,i])
            k=where(err_i NE missing,nk)
            IF nk NE 0 THEN BEGIN 
              npx = max([0.02*nk,min([nk,100])])
              idx  = (sort(wd_i[k]))[0:npx-1]
              dc   = median(wd_i[k[idx]])
            ENDIF ELSE BEGIN
              dc=0.
            ENDELSE 
           ; wd_i = wd[*,*,i]
           ; idx  = (sort(wd_i))[0:npx-1]
           ; dc   = median(wd_i[idx])
            wd[*,*,i] = (wd_i - dc) ; >0
            if verbose eq 1 then $
            message,"WINDOW = "+trim(iwin,'(i2.2)')+" NEXP = "+trim(i,'(i3.3)')+" DC = "+trim(dc,'(i3.3)'),/info
          endfor
        endelse
      endif else begin
        if iwin eq 0 or verbose then $
        message,"SLOT - USING PEDESTAL",/info
        dc = ([549.0,500.0,556.0,547.0])(quad)
        for i=0,data->getnexp()-1 do begin
          wd[*,*,i] = (wd[*,*,i] - dc); >0
          if verbose eq 1 then $
          message,"WINDOW = "+trim(iwin,'(i2.2)')+" NEXP = "+trim(i,'(i3.3)')+" DC = "+trim(dc,'(i3.3)'),/info
        endfor
      endelse

    end

    '1st_order': begin
       if iwin eq 0 then begin
         message,'CAREFUL, 1st order dark current subtraction has no variation with',/info
         message,'         ccd temperature',/info
       endif
       for i=0,data->getnexp()-1 do begin
         wd[*,*,i]=(wd[*,*,i]-(dc_data.coeff[0,quad]+dc_data.coeff[1,quad]*exp_dur[i])); >1
       endfor
                 end
    'full_ccd': begin  
       if pos[0] gt 2147 then begin
         dc=ccda
         pos[0]=pos[0]-2148
       endif else dc=ccdb
       for i=0,data->getnexp()-1 do begin
         wd[*,*,i] = (wd[*,*,i] - dc[pos[0]:pos[0]+pos[1]-1, pos[2]:pos[2]+pos[3]-1]) ; > 0
       endfor
                end
    else: begin
        message,'no such dc.version '+dc_data.version,/info        
          end
  endcase
  return, wd
end
