pro eis_cal::seteisdefaults,time_obs,slit_ind,init=init
  if n_elements(init) eq 1 then return
; set various instrument parameters, read $EIS_RESPONSE/cal_response.txt      
;
; absolute calibration
;
  self.aeff_version=3
  self.aeff_fileroot=self.respdir + 'EIS_EffArea_'
  self.sr_factor=(725./1.496e8)^2
  self.ergs_to_photons=6.626e-27*2.998e10*1.e8
  self.gain=6.3 ;Hiro 6.45 8/2/2007 changed to 6.3 by Hiro & Louisa 15/4/2007
  self.phot_to_elec=12398.5/3.65
  self.tau_sensitivity = 1894.0 ; updated from 2300.0 on 19/04/2011
;
;pointing info
;
  self.xmidmirrpos=1800    ; home position of the fine mirror, reset to 1800 from 1200 on 08/02/2007
  self.xmidcmirrpos=43703l ; home position of the coarse mirror (NB! unsigned int -> stored in a long)
  self.ymidslitpos=512     ; mid position on the slit (should be 511.5?)
  self.xoffsetut=-129.6    ; offset between AOCS-SOT FOV and EIS midpoint measured in units of 
  self.yoffsetut=-36.3     ; No longer XRT pixels, but arcsec - taken from The List of Solar-B "Hinode" 
                           ; wide FITS Keywords, Dec 25 2006 Masumi Shimojo 
  xrtpixel=1.004           ; 
  self.fmirr_ss = 0.1248   ; fine mirror step size in arcsec (Khalid says 0.12299)
                           ; number should be multiplied by 2 to get image motion 
                           ; (Hiro 0.1203 from observation 8/2/2007)
  self.cmirr_ss = 0.032862 ; coarse mirror step size in arcsec according to Hiro's measurment
                           ; after the first coarse mirror move on 22/01/2007
  self.yoff_sw = [-17.0,8.72e-2] ; yoffsets as linear function of wavelength Kamio-san and Hiro-san
  self.yoff_lw = [-3.49,7.68e-2] ; "Y-offsets of EIS spectra" 18/8/2008
  self.yoff_cmirr = [0.0,-13.7]
  self.cmirr_move_dates = [20060922L,20070122L]

  self.launch_date = '2006-09-22T21:36'

  self.slit_pointing_offset = [0.,0.,8.,0.] ; offset of 2 arcsec slit
;
;define wavelength scale courtesy of Charlie Brown cbrown@ssd5.nrl.navy.mil 16 November 2006.
;
  self.npix = 2148
  nscan = 50
  self.lambda0.a = 199.9389   ; reference wavelength, pixel 0 (including scan pixels detector A)
  self.lambda0.b = 166.131    ; reference wavelength, pixel 0 (including scan pixels detector B)
  self.disp.a = 0.022332      ; dispersion, detector A (long wavelength)
  self.disp.b = 0.022317      ; dispersion, detector B (short wavelength)
  self.dispsq.a = -1.329e-8   ; dispersion, quadratic term, in wavelength equation, detector A (long)
  self.dispsq.b = -1.268e-8   ; dispersion, quadratic term, in wavelength equation, detector B (short)

  ccd_length = findgen(self.npix) - nscan + self.npix - 2*nscan
  self.lambda.scale_a = self.lambda0.a + self.disp.a*ccd_length + self.dispsq.a*ccd_length*ccd_length
  ccd_length = findgen(self.npix) - nscan  
  self.lambda.scale_b = self.lambda0.b + self.disp.b*ccd_length + self.dispsq.b*ccd_length*ccd_length
  
;----------------------------------------------------------------------------------------------------
;  Read calrespfile containing up to date calibration data
;
  problem=0 ; problem? what problem?
;
; define variables in case they are not to be found in an (old) calrespfile
;
  xmidmirrpos=self.xmidmirrpos
  ymidslitpos=self.ymidslitpos
  xoffsetut=self.xoffsetut
  yoffsetut=self.yoffsetut
  fmirr_ss = self.fmirr_ss
  yoff_sw = self.yoff_sw
  yoff_lw = self.yoff_lw
;                                                     
  calrespfile=self.calrespfile
  info=file_info(getenv('EIS_RESPONSE')+path_sep()+calrespfile)
  if not info.exists then begin
    message,'there was some problem reading '+getenv('EIS_RESPONSE')+path_sep()+calrespfile,/info
    message,'using hard coded calibration parameters',/info
    return
  endif
;
; The lines below are substituted by the instruction that follows
; The execute instruction cannot be executed without an IDL license
;
; commands=rd_tfile(getenv('EIS_RESPONSE')+path_sep()+calrespfile,/nocomment)
; for i=0,n_elements(commands)-1 do begin
;   result=execute(commands[i])
;   if not result then problem=1
; endfor
  @eis_cal_response
;
; absolute calibration
;
  self.aeff_version=aeff_version
  self.aeff_fileroot=self.respdir + aeff_fileroot
  self.sr_factor=sr_factor
  self.ergs_to_photons=ergs_to_photons
  self.gain=gain
  self.gainerr=gainerr
  self.aeffrelerr=aeffrelerr
  self.phot_to_elec=phot_to_elec
;
;pointing info
;
  self.xmidmirrpos=xmidmirrpos
  self.ymidslitpos=ymidslitpos
  self.xoffsetut=xoffsetut
  self.yoffsetut=yoffsetut
  self.fmirr_ss = fmirr_ss 
;
;define wavelength scale
;
  self.npix = npix
  nscan=50
  self.lambda0.a = lambda0_a  ; reference wavelength, pixel 0 (including scan pixels detector A)
  self.lambda0.b = lambda0_b  ; reference wavelength, pixel 0 (including scan pixels detector B)
  self.disp.a = disp_a       ; dispersion, detector A (long wavelength)
  self.disp.b = disp_b       ; dispersion, detector B (short wavelength)
  self.dispsq.a = dispsq_a   ; dispersion, quadratic term, in wavelength equation, detector A (long)
  self.dispsq.b = dispsq_b   ; dispersion, quadratic term, in wavelength equation, detector B (short)

  ccd_length = findgen(self.npix) - nscan + self.npix - 2*nscan
  self.lambda.scale_a = self.lambda0.a + self.disp.a*ccd_length + self.dispsq.a*ccd_length*ccd_length
  ccd_length = findgen(self.npix) - nscan  
  self.lambda.scale_b = self.lambda0.b + self.disp.b*ccd_length + self.dispsq.b*ccd_length*ccd_length
;
;define wavelength scale according to data in eis_get_ccd_translation
;
  nscan=0
  coeffs = eis_get_ccd_translation(slit_ind , time = time_obs) 
  self.lambda0.b = coeffs[0].a0
  self.disp.b    = coeffs[0].a1
  self.dispsq.b  = coeffs[0].a2
  self.lambda0.a = coeffs[1].a0
  self.disp.a    = coeffs[1].a1
  self.dispsq.a  = coeffs[1].a2
  ccd_length = findgen(self.npix) - nscan + self.npix - 2*nscan
  self.lambda.scale_a = self.lambda0.a + self.disp.a*ccd_length + self.dispsq.a*ccd_length*ccd_length
  ccd_length = findgen(self.npix) - nscan
  self.lambda.scale_b = self.lambda0.b + self.disp.b*ccd_length + self.dispsq.b*ccd_length*ccd_length
;
  if problem then begin
    message,'there was some problem reading '+getenv('EIS_RESPONSE')+path_sep()+calrespfile,/info
    message,'using hard coded calibration parameters',/info
  endif

  return
end
